var Db = require('../db'),
	JsUtils = require('../jsUtils'),
	Step = require('step');

module.exports = function (collectionName, model) {
	// wonderfully transparent function that finds a singular record with an id
	this.load = function (id, callback, nomethods) {
		this.find({id: id}, function (err, result) {
			if (err) {
				callback(err);
			} else {
				if (result.length > 0) {
					callback(undefined, result[0]);
				} else {
					callback('Record does not exist');
				}
			}
		}, nomethods);
	};
	
	this.findAll = function (callback, nomethods) {
		this.find({}, callback, nomethods);
	};
	
	/*	Tries to find records that adhere to the specified parameters.
		This find tries to be inclusive as possible, if an object in the
		db has all (and more) of the parameters you search for, then it's
		returned as well.
	*/
	this.find = function (parameters, callback, nomethods) {
		// provide a little safety net for callbacks
		var safeCallback = function (err, result) {
			if (callback) {
				callback(err, result);
			}
		};
		
		var searchParams = JsUtils.flattenObject(parameters);
		
		Db.getDb().collection(collectionName, function (err, collection) {
			if (err) {
				safeCallback(err);
			} else {
				collection.find(searchParams, function (err, cursor) {
					cursor.toArray(function (err, docs) {
						if (err) {
							safeCallback(err);
						} else {
							if (docs.length < 1) {
								safeCallback(undefined, []);
							} else {
								var toLoad = [];
								if (nomethods === true) {
									safeCallback(undefined, toLoad);
								} else {
									Step(
										function () {
											var group = this.group();
											for (var x = 0, l = docs.length; x < l; x++) {
												toLoad[x] = new model(docs[x]);
												toLoad[x].loadRelations(group());
											}
										},
										function (err, result) {
											if (err) throw err;
											
											this(undefined, toLoad);
										},
										safeCallback
									);
								}
							}
						}
					});
				});
			}
		});
	};
}
