/*	saveable
	Calling this on an object will give it 2 new methods; save, delete.
	These allow you to store the object to the database almost transparently.
	
	Feel free to write your own database implementation.
*/
var Db = require('../db');
var JsUtils = require('../jsUtils');
var Step = require('step');
var Unique = require('../unique');
	
/*
	WARNING: Your object will be assigned an id attribute, if it does not already have one.
			 This is only done on save, so you have plenty of time to give it an id (if needed).
*/

module.exports = function (collectionName) {
	/*
		database methods, these follow a callback paradigm
		this is done because of the non-blocking nature of mongo
	*/
	// save this to the DB, doesnt matter if the fucker hasn't been inserted before
	this.save = function (callback) {
		// make object accessable to the callbacks
		var toSave = this.attributes,
			originalModel = this;
		
		// provide a little safety net for callbacks
		var safeCallback = function (err, result) {
			if (callback) {
				callback(err, result);
			}
		};
		
		var saveFunction = function () {
			// tag the object with a last changed attribute
			toSave.timeLastChanged = ('' + (new Date()).getTime());
			// check for created time created attribute
			if (!toSave.timeCreated) {
				toSave.timeCreated = toSave.timeLastChanged;
			}
			
			Db.getDb().collection(collectionName, function (err, collection) {
				if (err) {
					safeCallback(err);
				} else {
					collection.save(toSave, function (err) {
						if (err) {
							safeCallback(err);
						} else {
							if (callback) {
								originalModel.attributes = toSave;
								safeCallback(undefined, originalModel);
							}
						}
					});
				}
			});
		};
		
		// good god there's gotta be a better way to get a unique id
		if (!toSave.id) {
			// Need to fetch id so other functions will work
			getAllIds(collectionName, function (err, result) {
				if (err) {
					safeCallback('Serious error when creating id: ' + err);
				} else {
					toSave.id = Unique(result);
					
					saveFunction();
				}
			});
		} else {
			saveFunction();
		}
	};
	
	this.delete = function (callback) {
		var toDelete = this.attributes;
		var model = this;
		
		// provide a little safety net for callbacks
		var safeCallback = function (err, result) {
			if (callback) {
				callback(err, result);
			}
		};
		
		// check to see if this record has been save before
		if (toDelete._id === undefined) {
			// has not been saved before, can't delete!
			safeCallback(new Error('Cannot delete record, it does not exist!'));
		} else {
			Db.getDb().collection(collectionName, function (err, collection) {
				if (err) {
					safeCallback(err);
				} else {
					Step(
						function () {
							model.deleteRelations(this);
						},
						function (err) {
							if (err) throw err;
							collection.remove({_id: toDelete._id}, this);
						},
						safeCallback
					);
				}
			});
		}
	};
};

function getAllIds(collectionName, callback) {
	Db.getDb().collection(collectionName, function (err, collection) {
		if (err) {
			callback(err);
		} else {
			collection.find(function (err, cursor) {
				cursor.toArray(function (err, docs) {
					if (err) {
						callback(err);
					} else {
						var toReturn = {};
						
						var l = docs.length;
						for (var x = 0; x < l; x++) {
							if (docs[x].id) {
								toReturn[docs[x].id] = true;
							}
						}
						callback(undefined, toReturn);
					}
				});
			});
		}
	});
};
