var Api = require('./api'),
	_ = require('underscore');

module.exports = function (options) {
	if (!options.classObject) throw new Error('Cannot create RestApi without specifying a classObject');
	if (!options.classObject.model) throw new Error('Cannot create RestApi without specifying a model');
	if (!options.application) throw new Error('Cannot create RestApi without specifying an application');
	if (!options.path) throw new Error('Cannot create RestApi without specifying a path');

	var app = options.application;
	var classObject = options.classObject;
	var model = options.classObject.model
	var path = options.path + (/\/$/.test(options.path) ? '' : '/');

	var activeRoutes = {
		'create': true,
		'read': true,
		'update': true,
		'delete': true
	};
	_.each(options.disable, function (value) {
		if (activeRoutes[value] !== undefined) {
			activeRoutes[value] = false;
		}
	});
	
	// CREATE
	if (activeRoutes['create'])
	app.post(path, function (req, res) {
		var newObject = new model(req.body);
		newObject.save(function (err, result) {
			if (err) {
				Api.error(res, err);
			} else {
				result.toResponse(res);
			}
		});
	});

	// READ
	if (activeRoutes['read'])
	app.get(path + ':id', function (req, res) {
		classObject.load(req.params.id, function (err, result) {
			if (err) {
				Api.error(res, err);
			} else {
				result.toResponse(res);
			}
		});
	});

	// UPDATE
	if (activeRoutes['update'])
	app.put(path + ':id', function (req, res) {
		classObject.load(req.params.id, function (err, result) {
			if (err) {
				Api.error(res, err);
			} else {
				result.setAll(req.body);
				result.save(function (err, result) {
					if (err) {
						Api.error(res, err);
					} else {
						result.toResponse(res);
					}
				});
			}
		});
	});

	// DELETE
	if (activeRoutes['delete'])
	app.del(path + ':id', function (req, res) {
		classObject.load(req.params.id, function (err, result) {
			if (err) {
				Api.error(res, err);
			} else {
				result.delete(function (err, result) {
					if (err) {
						Api.error(res, err);
					} else {
						Api.success(res);
					}
				});
			}
		});
	});

	_.each(options.customRoutes, function (value) {
		console.log('adding custom route for', value);
		var routePath = (value.path !== undefined) ? value.path : value.method;
		var bindFunction;
		if (value.type === 'GET') {
			bindFunction = app.get;
		} else if (value.type === 'POST') {
			bindFunction = app.post;
		} else if (value.type === 'PUT') {
			bindFunction = app.put;
		} else if (value.type === 'DELETE') {
			bindFunction = app.del;
		}
		
		if (value.context === 'model') {
			bindFunction.call(app, path + ':id/' + routePath, function (req, res) {
				classObject.load(req.params.id, function (err, result) {
					if (err) {
						Api.error(res, err);
					} else {
						result[value.method](req, res);
					}
				});
			});
		} else if (value.context === 'class') {
			bindFunction.call(app, path + routePath, classObject[value.method]);
		} else {
			throw new Error('Context for custom route unknown');
		}
	});
};
