// config
global.modlr = {
	database: 'modlr_tests'
};

var NUM_BLOGS = 5;
var NUM_COMMENTS = {};
var BLOGS = [];
var DELETED;


var Step = require('step');
var _ = require('underscore');
var Db = require('../../db');
var BlogClass = require('./models/blog');
var CommentClass = require('./models/comment');

Step(
	// Setup database
	function () {
		Db.getDb().dropDatabase(this);
	},
	// populate blogs
	function (err, result) {
		if (err) throw err;
		
		var saveFunctions = [];
		for (var x = 0; x < NUM_BLOGS; x++) {
			saveFunctions.push(function () {
				var blog = new BlogClass.model();
				BLOGS.push(blog);
				blog.save(this);
			});
		}
		
		saveFunctions.push(this);
		Step.apply(undefined, saveFunctions);
	},
	// populate comments
	function (err) {
		if (err) throw err;
		
		var saveFunctions = [];
		_.each(BLOGS, function (value) {
			var id = value.get('id');
			
			NUM_COMMENTS[id] = Math.floor(Math.random() * 10);
			for (var x = 0; x < NUM_COMMENTS[id]; x++) {
				saveFunctions.push(function () {
					var comment = new CommentClass.model({blogId: id});
					comment.save(this);
				});
			}
		});
		
		saveFunctions.push(this);
		Step.apply(this, saveFunctions);
	},
	// load the 3rd blog
	function (err) {
		if (err) throw err;
		BlogClass.load(BLOGS[2].get('id'), this);
	},
	// delete it
	function (err, result) {
		if (err) throw err;
		
		DELETED = BLOGS.splice(2, 1)[0];
		result.delete(this);
	},
	// fetch comments for deleted blog
	function (err) {
		if (err) throw err;
		CommentClass.find({blogId: DELETED.get('id')}, this);
	},
	// assert that existing
	function (err, results) {
		if (err) throw err;
		
		console.log('Asserting...');
		
		if (results.length > 0) {
			throw new Error('Comments were not deleted on cascading delete');
		}
		
		console.log('Passed: parental model relationship delete');
		
		// clean up
		Db.getDb().dropDatabase(function () {
			process.exit(0);
		});
	}
	
);