// config
global.modlr = {
	database: 'modlr_tests'
};

var NUM_BLOGS = 5;
var NUM_COMMENTS = {};
var BLOGS = [];


var Step = require('step');
var _ = require('underscore');
var Db = require('../../db');
var BlogClass = require('./models/blog');
var CommentClass = require('./models/comment');

Step(
	// Setup database
	function () {
		Db.getDb().dropDatabase(this);
	},
	// populate blogs
	function (err, result) {
		if (err) throw err;
		
		var saveFunctions = [];
		for (var x = 0; x < NUM_BLOGS; x++) {
			saveFunctions.push(function () {
				var blog = new BlogClass.model();
				BLOGS.push(blog);
				blog.save(this);
			});
		}
		
		saveFunctions.push(this);
		Step.apply(undefined, saveFunctions);
	},
	// populate comments
	function (err) {
		if (err) throw err;
		
		var saveFunctions = [];
		_.each(BLOGS, function (value) {
			var id = value.get('id');
			
			NUM_COMMENTS[id] = Math.floor(Math.random() * 10);
			for (var x = 0; x < NUM_COMMENTS[id]; x++) {
				saveFunctions.push(function () {
					var comment = new CommentClass.model({blogId: id});
					comment.save(this);
				});
			}
		});
		
		saveFunctions.push(this);
		Step.apply(this, saveFunctions);
	},
	// fetch comments
	function (err) {
		
		if (err) throw err;
		
		var group = this.group();
		_.each(BLOGS, function (value) {
			BlogClass.load(value.get('id'), group());
		});
	},
	// assert
	function (err, results) {
		if (err) throw err;
		
		console.log('Asserting...');
		
		_.each(results, function (value) {
			if (value.get('comments').length !== NUM_COMMENTS[value.get('id')]) {
				throw new Error(value.get('id') + ' has ' + value.get('comments').length +
				' comments, when it should have: ' + NUM_COMMENTS[value.get('id')]);
			}
		});
		
		console.log('Passed: parental model relationship load');
		
		// clean up
		Db.getDb().dropDatabase(function () {
			process.exit(0);
		});
		
	}
	
);